import pytest
from ansiconverter.converter import *
# from ansiconverter import styles
from ansiconverter.styles import styles


# Tests for the converter module


def test_RGBtoHEX():
    assert RGBtoHEX([13, 24, 128]) == "#0d1880"


def test_RGBtoANSI_blank_list():
    with pytest.raises(ValueError) as e_info:
        RGBtoANSI("text", [])


def test_RgbtoAnsi_blank():
    assert RGBtoANSI("text") == "\x1B[38;2;255;255;255mtext\x1b[0m"


def test_HEXtoRGB():
    assert HEXtoRGB("#1b815a") == [27, 129, 90]


def test_HEXtoRGB_shorterValue():
    with pytest.raises(ValueError) as e_info:
        HEXtoRGB("#1b815")


def test_HEXtoRGB_longerValue():
    with pytest.raises(ValueError) as e_info:
        HEXtoRGB("#1b815ad")


def test_HEXtoRGB_2sharp():
    assert HEXtoRGB("##1b815a") == [27, 129, 90]


# Tests for the styles module


def test_styles_bold():
    assert styles.bold("Test") == "\x1b[1mTest\x1b[0m"


def test_styles_faint():
    assert styles.faint("Test") == "\x1b[2mTest\x1b[0m"


def test_styles_italic():
    assert styles.italic("Test") == "\x1b[3mTest\x1b[0m"



from ansiconverter.converter import *


class styles:
    """Apply different styles to the input text.

    Available :
        - bold
        - faint
        - italic
        - underline
        - bold & underline
        - strikethrough
        - reverse
    """

    RESET = "\x1b[0m"

    def bold(text):
        return f"\033[1m{text}{styles.RESET}"

    def faint(text):
        return "\033[2m" + text + styles.RESET

    def italic(text):
        return "\033[3m" + text + styles.RESET

    def underline(text):
        return "\033[4m" + text + styles.RESET

    def bold_and_underline(text):
        return "\033[1;4m" + value + styles.RESET

    def strikethrough(text):
        return "\033[9m" + text + styles.RESET

    def reverse(text):
        return "\033[7m" + text + styles.RESET


class bootstrap_inspired:
    pass



""" Convert any colour to the ANSI format to write in colours in your terminal.
Note: The conversion to an ANSI escape sequence may induce some colour variations. 
Also notice that some colours can't be mixed together as foreground and background.
"""


RESET = "\x1b[0m"


def RGBtoANSI(text: str, foregound=[255, 255, 255], background=[],):
    """Write a text in RGB colour.

    Args:
        text (str): the text you want to write.
        foregound (list, optional): RGB foregound's colour. Defaults to [255, 255, 255].
        background (list, optional): RGB background's colour. Defaults to [].

    Raises:
        ValueError: the foregound colour can't be an empty list.

    Returns:
        string: the ANSI code for the foreground and the background.
    """
    if foregound != []:
        if background == []:
            return f"\033[38;2;{foregound[0]};{foregound[1]};{foregound[2]}m{str(text)}{RESET}"
        else:
            return f"\033[38;2;{foregound[0]};{foregound[1]};{foregound[2]}m\033[48;2;{background[0]};{background[1]};{background[2]}m{str(text)}{RESET}"
    else:
        raise ValueError(
            "The foregound can't be an empty list!\nNo paramaters will write the text in write"
        )


def HEXtoRGB(fg="#000000"):
    """Convert a hexadecimal colour to its RGB triplet.

    Args:
        fg (str, optional): Hexadecimal colour value. Defaults to "#000000".

    Raises:
        ValueError: The colour is not a correct hexadecimal value.

    Returns:
        list: triplet of RGB values.
    """
    foreground = ""
    while True:
        foreground = fg.lstrip("#").lower()
        if len(foreground) == 6:
            return list(int(foreground[i: i + 2], base=16) for i in (0, 2, 4))

        else:
            raise ValueError("Enter a valid hexadecimal value")
            return 1


def HEXtoANSI(text, foreground="#ffffff", background=""):
    from ansiconverter.converter import HEXtoRGB

    foregroundRGB = HEXtoRGB(foreground)
    if foreground != "":
        if background != "":
            backgroundRGB = HEXtoRGB(background)
            return RGBtoANSI(text, foregroundRGB, backgroundRGB)
        else:
            return RGBtoANSI(text, foregroundRGB)

    else:
        raise ValueError("Please enter at least one foreground colour.")


def RGBtoHEX(rgb=[255, 255, 255]):
    """Convert any RGB colour to hexadecimal.

    Args:
        rgb (list, optional): the colour you want to convert. Defaults to [255, 255, 255].

    Returns:
        string: colour's hexadecimal value
    """
    if rgb != []:
        return f"#{rgb[0]:02x}{rgb[1]:02x}{rgb[2]:02x}"
    else:
        raise (ValueError(f"The colour can't be an empty list. Please retry."))



from setuptools import setup, find_packages

with open("README.md", "r") as f:
    long_description = f.read()

setup(
    name="ansiconverter",
    version="1.0.0",
    description="Convert any color in RGB or hexadecimal format to ANSI code",
    license="MIT",
    long_description=long_description,
    long_description_content_type="text/markdown",
    # py_packages=["ansiconverter","converter"],
    packages=find_packages(),
    author="Thomas Beyet",
    author_email=None,
    url="https://github.com/thomassamoth/ansiconverter",
    # package_dir={"": "ansiconverter"},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    extras_require={
        "dev": [
            "pytest>=3.6",
        ],
    },
    project_urls={
        "source": "http://github.com/thomassamoth/ansiconverter",
    },
    python_requires=">=3.6",
)
